@@ -33,8 +33,7 @@
 Base Skeleton Graph.
 
 Simple example usage:
->>> import matplotlib.pyplot as pyplot
->>> import numpy
+>>> # --- Object
 >>> import skimage.data as data
 >>> import skimage.util as util
 >>> object_map = util.invert(data.horse())
@@ -44,54 +43,51 @@ Simple example usage:
 >>> # --- SKL Graph
 >>> from skl_graph.skl_graph import skl_graph_t
 >>> skl_graph = skl_graph_t.FromSKLMap(skl_map)
->>> # --- End of SKLGraph-based sections
->>> _, axes = pyplot.subplots(ncols=3)
->>> axes[0].matshow(object_map, cmap="gray")
->>> axes[1].matshow(skl_map, cmap="gray")
->>> skl_graph.Plot(axes=axes[2], should_block=False)
->>> for ax, title in zip(axes, ("Object", "Skeleton", "Graph")):
->>>     ax.set_title(title)
->>>     ax.set_axis_off()
+>>> # --- Plotting
+>>> import matplotlib.pyplot as pyplot
+>>> _, all_axes = pyplot.subplots(ncols=3)
+>>> all_axes[0].matshow(object_map, cmap="gray")
+>>> all_axes[1].matshow(skl_map, cmap="gray")
+>>> from skl_graph.task.plot.skl_graph import Plot
+>>> Plot(axes=all_axes[2], should_block=False)
+>>> for axes, title in zip(all_axes, ("Object", "Skeleton", "Graph")):
+>>>     axes.set_title(title)
+>>>     axes.set_axis_off()
 >>> pyplot.tight_layout()
 >>> pyplot.show()
 """
 
 from __future__ import annotations
 
-from typing import Callable, ClassVar, Dict, List, Optional, Sequence, Tuple, Union
+from typing import ClassVar, Dict, List, Sequence, Tuple
 
-import matplotlib.pyplot as pl_
 import networkx as nx_
-import numpy as np_
+import numpy as nmpy
 import scipy.ndimage as im_
 import skimage.draw as dw_
-from mpl_toolkits import mplot3d as m3_
 
-import skl_graph.brick.edge as dg_
-import skl_graph.brick.node as nd_
-import skl_graph.brick.topology_map as bymp
+import skl_graph.type.edge as dg_
+import skl_graph.type.node as nd_
+import skl_graph.type.topology_map as bymp
 from skl_graph.brick.constants import UNTESTED_VALIDITY
-from skl_graph.brick.plot import (
-    axes_t,
+from skl_graph.type.plot import (
+    direction_style_raw_h,
     direction_style_t,
-    direction_style_unstructured_h,
-    edge_style_unstructured_h,
+    edge_style_raw_h,
+    edge_style_t,
     edge_styles_h,
-    edge_styles_unstructured_h,
-    figure_t,
+    edge_styles_raw_h,
+    label_style_raw_h,
     label_style_t,
-    label_style_unstructured_h,
     label_styles_h,
-    label_styles_unstructured_h,
-    node_style_unstructured_h,
+    label_styles_raw_h,
+    node_either_raw_h,
+    node_style_t,
     node_styles_h,
-    node_styles_unstructured_h,
-    plot_mode_e,
-    plot_style_t,
 )
 
 
-array_t = np_.ndarray
+array_t = nmpy.ndarray
 
 
 class skl_graph_t(nx_.MultiGraph):
@@ -172,7 +168,7 @@ class skl_graph_t(nx_.MultiGraph):
         for cmp_label in range(1, n_components + 1):
             if n_components > 1:
                 cmp_map = cc_map == cmp_label
-                single_tmap = np_.full_like(tmap, background_label)
+                single_tmap = nmpy.full_like(tmap, background_label)
                 single_tmap[cmp_map] = tmap[cmp_map]
             else:
                 cmp_map = skl_map
@@ -208,19 +204,10 @@ class skl_graph_t(nx_.MultiGraph):
                 instance.n_e_nodes += e_nodes.__len__()
                 instance.n_b_nodes += b_nodes.__len__()
 
-        instance.node_styles = {
-            None: plot_style_t(type=".", size=6, color="g"),
-            1: plot_style_t(type=".", size=6, color="r"),
-        }
-        instance.edge_styles = (
-            plot_style_t(type="-", size=2, color="k"),
-            plot_style_t(type=":", size=2, color="k"),
-        )
-        instance.direction_style = direction_style_t(show=False, size=0, color="k")
-        instance.label_styles = (
-            label_style_t(show=True, size=6.0, color="k"),
-            label_style_t(show=False, size=6.0, color="k"),
-        )
+        instance.node_styles = node_style_t.Default()
+        instance.edge_styles = edge_style_t.Default()
+        instance.direction_style = direction_style_t.Default()
+        instance.label_styles = label_style_t.Default()
 
         return instance
 
@@ -240,10 +227,10 @@ class skl_graph_t(nx_.MultiGraph):
         -------
 
         """
-        singleton = np_.where(tmap == 0)
+        singleton = nmpy.where(tmap == 0)
         if singleton[0].size > 0:
             # Can only be 1 since tmap is processed per connected components (*)
-            singleton = np_.array(singleton, dtype=np_.int64).squeeze()
+            singleton = nmpy.array(singleton, dtype=nmpy.int64).squeeze()
             end_node = nd_.end_node_t.WithPosition(singleton, width_map=width_map)
             self.add_node(end_node.uid, as_node_t=end_node)
 
@@ -253,14 +240,14 @@ class skl_graph_t(nx_.MultiGraph):
         #
         elif (tmap[tmap != background_label] == 2).all():
             # The tmap represents a self loop
-            loop_slc = np_.nonzero(tmap == 2)
+            loop_slc = nmpy.nonzero(tmap == 2)
 
             # Takes the first pixel to serve as a node, and the rest for the self-loop edge
             #
             # 0:1 makes sites elements array_t's (instead of numpy numbers), which is necessary for
             # nd_.branch_node_t.WithCentroidAndSites, but requires to squeeze the centroid.
             sites = tuple(per_dim[0:1] for per_dim in loop_slc)
-            centroid = np_.array(sites, dtype=np_.float64).squeeze()
+            centroid = nmpy.array(sites, dtype=nmpy.float64).squeeze()
             node = nd_.branch_node_t.WithCentroidAndSites(
                 centroid, sites=sites, width_map=width_map
             )
@@ -391,8 +378,8 @@ class skl_graph_t(nx_.MultiGraph):
         topological_map[topological_map == bymp.TMapBackgroundLabel(skl_map)] = 0
         topological_map[topological_map > 3] = 3
 
-        binary_correctness = np_.array_equal(rebuilt_skl_map > 0, skl_map)
-        topology_correctness = np_.array_equal(rebuilt_skl_map, topological_map)
+        binary_correctness = nmpy.array_equal(rebuilt_skl_map > 0, skl_map)
+        topology_correctness = nmpy.array_equal(rebuilt_skl_map, topological_map)
 
         return binary_correctness, topology_correctness
 
@@ -401,12 +388,12 @@ class skl_graph_t(nx_.MultiGraph):
         if (not self.has_widths) and with_width:
             with_width = False
         if with_width:
-            dtype = np_.float64
+            dtype = nmpy.float64
         else:
             # Not uint to allow for subtraction
-            dtype = np_.int8
+            dtype = nmpy.int8
 
-        output = np_.zeros(self.domain_lengths, dtype=dtype)
+        output = nmpy.zeros(self.domain_lengths, dtype=dtype)
 
         for ___, ___, edge in self.edges.data("as_edge_t"):
             if with_width:
@@ -434,7 +421,7 @@ class skl_graph_t(nx_.MultiGraph):
             raise ValueError("Requires an SKL graph with widths")
 
         # Not uint to allow for subtraction
-        output = np_.zeros(self.domain_lengths, dtype=np_.int8)
+        output = nmpy.zeros(self.domain_lengths, dtype=nmpy.int8)
 
         if self.dim == 2:
             NewBall = dw_.disk
@@ -445,15 +432,15 @@ class skl_graph_t(nx_.MultiGraph):
             if isinstance(node, nd_.branch_node_t):
                 for *sites, radius in zip(
                     *node.sites,
-                    np_.around(0.5 * (node.diameters - 1.0)).astype(np_.int64),
+                    nmpy.around(0.5 * (node.diameters - 1.0)).astype(nmpy.int64),
                 ):
                     output[NewBall(sites, radius, shape=output.shape)] = 1
             else:
                 output[
                     NewBall(
                         node.position,
-                        np_.around(0.5 * (node.diameter - 1.0))
-                        .astype(np_.int64)
+                        nmpy.around(0.5 * (node.diameter - 1.0))
+                        .astype(nmpy.int64)
                         .item(),
                         shape=output.shape,
                     )
@@ -461,7 +448,7 @@ class skl_graph_t(nx_.MultiGraph):
 
         for ___, ___, edge in self.edges.data("as_edge_t"):
             for *sites, radius in zip(
-                *edge.sites, np_.around(0.5 * (edge.widths - 1.0)).astype(np_.int64)
+                *edge.sites, nmpy.around(0.5 * (edge.widths - 1.0)).astype(nmpy.int64)
             ):
                 output[NewBall(sites, radius, shape=output.shape)] = 1
 
@@ -471,16 +458,19 @@ class skl_graph_t(nx_.MultiGraph):
         self,
         /,
         *,
-        node: Union[node_style_unstructured_h, node_styles_unstructured_h] = None,
-        edge: edge_style_unstructured_h = None,
-        regular_edge: edge_style_unstructured_h = None,
-        self_loop: edge_style_unstructured_h = None,
-        edges: edge_styles_unstructured_h = None,
-        direction: direction_style_unstructured_h = None,
-        label: label_style_unstructured_h = None,
-        label_node: label_style_unstructured_h = None,
-        label_edge: label_style_unstructured_h = None,
-        labels: label_styles_unstructured_h = None,
+        node: node_either_raw_h = None,
+        #
+        edge: edge_style_raw_h = None,
+        regular_edge: edge_style_raw_h = None,
+        self_loop: edge_style_raw_h = None,
+        edges: edge_styles_raw_h = None,
+        #
+        direction: direction_style_raw_h = None,
+        #
+        label: label_style_raw_h = None,
+        label_node: label_style_raw_h = None,
+        label_edge: label_style_raw_h = None,
+        labels: label_styles_raw_h = None,
     ) -> None:
         """
         edge: common style for both regulars and self-loops
@@ -489,15 +479,10 @@ class skl_graph_t(nx_.MultiGraph):
         labels: styles for nodes and edges, in that order
         """
         if node is not None:
-            if isinstance(node, tuple):  # node_style_unstructured_h
-                self.node_styles = {
-                    None: plot_style_t(type=node[0], size=node[1], color=node[2])
-                }
+            styles = node_style_t.NewFromUnstructured(node)
+            if isinstance(node, tuple):  # node_style_raw_h
+                self.node_styles = styles
             else:
-                styles = {
-                    _dgr: plot_style_t(type=_stl[0], size=_stl[1], color=_stl[2])
-                    for _dgr, _stl in node.items()
-                }
                 self.node_styles.update(styles)
             if None not in self.node_styles:
                 # Should never happen, but just in case...
@@ -506,41 +491,18 @@ class skl_graph_t(nx_.MultiGraph):
                     f'Expected: style dictionary with a "None" entry.'
                 )
 
-        if edge is not None:
-            style = plot_style_t(type=edge[0], size=edge[1], color=edge[2])
-            styles = (style, style)
-        elif regular_edge is not None:
-            style = plot_style_t(
-                type=regular_edge[0],
-                size=regular_edge[1],
-                color=regular_edge[2],
-            )
-            styles = (style, self.edge_styles[1])
-        elif self_loop is not None:
-            style = plot_style_t(
-                type=self_loop[0],
-                size=self_loop[1],
-                color=self_loop[2],
-            )
-            styles = (self.edge_styles[0], style)
-        elif edges is not None:
-            styles = tuple(
-                plot_style_t(
-                    type=edges[_idx][0],
-                    size=edges[_idx][1],
-                    color=edges[_idx][2],
-                )
-                for _idx in range(2)
-            )
-        else:
-            styles = None
+        styles = edge_style_t.AllFromUnstructured(
+            self.edge_styles,
+            edge=edge,
+            regular_edge=regular_edge,
+            self_loop=self_loop,
+            edges=edges,
+        )
         if styles is not None:
             self.edge_styles = styles
 
         if direction is not None:
-            self.direction_style = direction_style_t(
-                show=direction[0], size=direction[1], color=direction[2]
-            )
+            self.direction_style = direction_style_t.NewFromUnstructured(direction)
 
         if label is not None:
             style = label_style_t(show=label[0], size=label[1], color=label[2])
@@ -573,231 +535,6 @@ class skl_graph_t(nx_.MultiGraph):
         if styles is not None:
             self.label_styles = styles
 
-    def Plot(
-        self,
-        figure: pl_.Figure = None,
-        axes: pl_.axes.Axes = None,
-        mode: plot_mode_e = plot_mode_e.SKL,
-        max_distance: float = 1.0,
-        colormap: Dict[int, str] = None,
-        should_block: bool = True,
-        should_return_figure: bool = False,
-        should_return_axes: bool = False,
-    ) -> Optional[Union[figure_t, axes_t, Tuple[figure_t, axes_t]]]:
-        #
-        if self.number_of_nodes() < 1:
-            print(f"{__name__}.{self.Plot.__name__}: Empty graph")
-            return None
-
-        if axes is None:
-            if figure is None:
-                figure = pl_.figure()
-            if self.dim == 2:
-                axes = figure.gca()
-            else:
-                axes = figure.add_subplot(1, 1, 1, projection=m3_.Axes3D.name)
-            axes.invert_yaxis()
-        else:
-            figure = axes.get_figure()
-
-        if axes.yaxis_inverted():
-            transformation = lambda y: y
-            vector_transf = lambda y: y
-        else:
-            max_0 = self.domain_lengths[0] - 1
-            transformation = lambda y: max_0 - np_.asarray(y)
-            vector_transf = lambda y: -np_.asarray(y)
-
-        transform_coords = lambda pos: (pos[1], transformation(pos[0]), *pos[2:])
-        positions_as_dict = dict(
-            (uid, transform_coords(node.position))
-            for uid, node in self.nodes.data("as_node_t")
-        )
-
-        if self.dim == 2:
-            if mode is plot_mode_e.Networkx:
-                self._PlotWithNetworkX(positions_as_dict, axes, colormap)
-            #
-            elif mode in (
-                plot_mode_e.SKL,
-                plot_mode_e.SKL_Polyline,
-                plot_mode_e.SKL_Curve,
-            ):
-                self._PlotExplicitly(
-                    positions_as_dict,
-                    transformation,
-                    vector_transf,
-                    axes,
-                    mode,
-                    max_distance,
-                )
-            #
-            elif mode is plot_mode_e.Graphviz:
-                self._PlotWithGraphviz(axes)
-            #
-            else:
-                raise ValueError(f"{mode}: Invalid plotting mode")
-            #
-        else:
-            self._PlotExplicitly(
-                positions_as_dict,
-                transformation,
-                vector_transf,
-                axes,
-                mode,
-                max_distance,
-            )
-
-        if self.dim == 2:
-            # Matplotlib says: NotImplementedError: It is not currently possible to manually set the aspect on 3D axes
-            axes.axis("equal")
-
-        if should_block:
-            pl_.show()  # Better named as TriggerMatplotlibEventLoop
-            return None
-        elif should_return_figure:
-            if should_return_axes:
-                return figure, axes
-            else:
-                return figure
-        elif should_return_axes:
-            return axes
-
-        return None
-
-    def _PlotWithNetworkX(
-        self,
-        positions_as_dict: Dict[str, Tuple[int, ...]],
-        axes: pl_.axes.Axes,
-        colormap: Optional[Dict[int, str]],
-    ) -> None:
-        #
-        if colormap is None:
-            colormap = self.__class__.colormap
-
-        node_degrees = (elm[1] for elm in self.degree)
-        node_colors = tuple(
-            colormap[degree] if degree < 3 else colormap[2] for degree in node_degrees
-        )
-
-        nx_.draw_networkx(
-            self,
-            ax=axes,
-            pos=positions_as_dict,
-            node_color=node_colors,
-            with_labels=self.label_styles[0].show,
-            font_size=int(round(self.label_styles[0].size)),
-            width=self.edge_styles[0].size,
-        )
-        if self.label_styles[1].show:
-            nx_.draw_networkx_edge_labels(
-                self,
-                ax=axes,
-                pos=positions_as_dict,
-                edge_labels=self._EdgeIDsForPlot(),
-                font_size=int(round(self.label_styles[1].size)),
-            )
-
-    def _PlotExplicitly(
-        self,
-        positions_as_dict: Dict[str, Tuple[int, ...]],
-        transformation: Callable[[array_t], array_t],
-        vector_transf: Callable[[array_t], array_t],
-        axes: pl_.axes.Axes,
-        mode: plot_mode_e,
-        max_distance: float,
-    ) -> None:
-        #
-        if mode == plot_mode_e.SKL:
-            mode = "site"
-        elif mode == plot_mode_e.SKL_Polyline:
-            mode = "polyline"
-        else:
-            mode = "curve"
-        dg_.Plot(
-            self.edges.data("as_edge_t"),
-            transformation,
-            vector_transf,
-            axes,
-            self.edge_styles,
-            self.direction_style,
-            self.label_styles[1],
-            mode=mode,
-            max_distance=max_distance,
-        )
-        # TODO: there is no distinction between end nodes and isolated nodes
-        nd_.PlotEndNodes(
-            self.nodes.data("as_node_t"),
-            transformation,
-            axes,
-            self.node_styles.get(1, self.node_styles[None]),
-        )
-
-        if self.dim == 2:
-            nd_.Plot2DBranchNodes(
-                self.nodes.data("as_node_t"),
-                self.degree,
-                transformation,
-                axes,
-                self.node_styles,
-            )
-            if self.label_styles[0].show:
-                nx_.draw_networkx_labels(
-                    self,
-                    ax=axes,
-                    pos=positions_as_dict,
-                    font_size=int(round(self.label_styles[0].size)),
-                    font_color=self.label_styles[0].color,
-                )
-        else:
-            nd_.Plot3DBranchNodes(
-                self.nodes.data("as_node_t"),
-                self.degree,
-                transformation,
-                axes,
-                self.node_styles,
-            )
-            if self.label_styles[0].show:
-                nd_.Plot3DNodeLabels(self, positions_as_dict, axes, self.label_styles)
-
-    def _PlotWithGraphviz(self, axes: pl_.axes.Axes) -> None:
-        #
-        try:
-            import tempfile as tp_
-
-            import imageio as io_
-            import pygraphviz as gp_
-
-            graph = nx_.nx_agraph.to_agraph(self)
-            with tp_.NamedTemporaryFile() as tmp_accessor:
-                img_name = tmp_accessor.name
-                graph.layout()
-                graph.draw(img_name, format="png")
-                axes.imshow(io_.v3.imread(img_name))
-        except Exception as exc:
-            axes.text(
-                0,
-                0,
-                f"Unable to plot graph using pygraphviz/imageio.\nPlease check installed modules.\n[{exc}]",
-                horizontalalignment="center",
-            )
-
-    def _EdgeIDsForPlot(self) -> Dict[str, str]:
-        #
-        lengths_as_dict = nx_.get_edge_attributes(self, "length")
-        w_lengths_as_dict = (
-            nx_.get_edge_attributes(self, "w_length") if self.has_widths else None
-        )
-
-        w_length_str = ""
-        edge_ids = {}
-        for key, value in lengths_as_dict.items():
-            if w_lengths_as_dict is not None:
-                w_length_str = "/" + str(round(w_lengths_as_dict[key]))
-            edge_ids[key[0:2]] = key[2] + "\n" + str(round(value)) + w_length_str
-
-        return edge_ids
-
     def __str__(self) -> str:
         """"""
         output = (
@@ -815,7 +552,7 @@ class skl_graph_t(nx_.MultiGraph):
 
 def _Ball3D(center: Sequence[int], radius: int, shape: Tuple[int, int, int]) -> array_t:
     #
-    output = np_.zeros(shape, dtype=np_.bool_)
+    output = nmpy.zeros(shape, dtype=nmpy.bool_)
     # dw_.ellipsoid leaves a one pixel margin around the ellipse, hence [1:-1, 1:-1, 1:-1]
     ellipse = dw_.ellipsoid(radius, radius, radius)[1:-1, 1:-1, 1:-1]
     sp_slices = tuple(
